@@ -1,3 +1,4 @@
+"""Debugger implementation for the IPython kernel."""
 import os
 import re
 import sys
@@ -30,19 +31,33 @@ try:
     _is_debugpy_available = True
 except ImportError:
     _is_debugpy_available = False
+except Exception as e:
+    # We cannot import the module where the DebuggerInitializationError
+    # is defined
+    if e.__class__.__name__ == "DebuggerInitializationError":
+        _is_debugpy_available = False
+    else:
+        raise e
+
 
 # Required for backwards compatiblity
 ROUTING_ID = getattr(zmq, "ROUTING_ID", None) or zmq.IDENTITY
 
 
 class _FakeCode:
+    """Fake code class."""
+
     def __init__(self, co_filename, co_name):
+        """Init."""
         self.co_filename = co_filename
         self.co_name = co_name
 
 
 class _FakeFrame:
+    """Fake frame class."""
+
     def __init__(self, f_code, f_globals, f_locals):
+        """Init."""
         self.f_code = f_code
         self.f_globals = f_globals
         self.f_locals = f_locals
@@ -50,28 +65,37 @@ class _FakeFrame:
 
 
 class _DummyPyDB:
+    """Fake PyDb class."""
+
     def __init__(self):
+        """Init."""
         from _pydevd_bundle.pydevd_api import PyDevdAPI
 
         self.variable_presentation = PyDevdAPI.VariablePresentation()
 
 
 class VariableExplorer:
+    """A variable explorer."""
+
     def __init__(self):
+        """Initialize the explorer."""
         self.suspended_frame_manager = SuspendedFramesManager()
         self.py_db = _DummyPyDB()
         self.tracker = _FramesTracker(self.suspended_frame_manager, self.py_db)
         self.frame = None
 
     def track(self):
+        """Start tracking."""
         var = get_ipython().user_ns
         self.frame = _FakeFrame(_FakeCode("<module>", get_file_name("sys._getframe()")), var, var)
         self.tracker.track("thread1", pydevd_frame_utils.create_frames_list_from_frame(self.frame))
 
     def untrack_all(self):
+        """Stop tracking."""
         self.tracker.untrack_all()
 
     def get_children_variables(self, variable_ref=None):
+        """Get the child variables for a variable reference."""
         var_ref = variable_ref
         if not var_ref:
             var_ref = id(self.frame)
@@ -80,6 +104,7 @@ class VariableExplorer:
 
 
 class DebugpyMessageQueue:
+    """A debugpy message queue."""
 
     HEADER = "Content-Length: "
     HEADER_LENGTH = 16
@@ -87,6 +112,7 @@ class DebugpyMessageQueue:
     SEPARATOR_LENGTH = 4
 
     def __init__(self, event_callback, log):
+        """Init the queue."""
         self.tcp_buffer = ""
         self._reset_tcp_pos()
         self.event_callback = event_callback
@@ -112,6 +138,7 @@ class DebugpyMessageQueue:
             self.message_queue.put_nowait(msg)
 
     def put_tcp_frame(self, frame):
+        """Put a tcp frame in the queue."""
         self.tcp_buffer += frame
 
         self.log.debug("QUEUE - received frame")
@@ -158,11 +185,15 @@ class DebugpyMessageQueue:
                 self._reset_tcp_pos()
 
     async def get_message(self):
+        """Get a message from the queue."""
         return await self.message_queue.get()
 
 
 class DebugpyClient:
+    """A client for debugpy."""
+
     def __init__(self, log, debugpy_stream, event_callback):
+        """Initialize the client."""
         self.log = log
         self.debugpy_stream = debugpy_stream
         self.event_callback = event_callback
@@ -229,6 +260,7 @@ class DebugpyClient:
         return attach_rep
 
     def get_host_port(self):
+        """Get the host debugpy port."""
         if self.debugpy_port == -1:
             socket = self.debugpy_stream.socket
             socket.bind_to_random_port("tcp://" + self.debugpy_host)
@@ -239,10 +271,12 @@ class DebugpyClient:
         return self.debugpy_host, self.debugpy_port
 
     def connect_tcp_socket(self):
+        """Connect to the tcp socket."""
         self.debugpy_stream.socket.connect(self._get_endpoint())
         self.routing_id = self.debugpy_stream.socket.getsockopt(ROUTING_ID)
 
     def disconnect_tcp_socket(self):
+        """Disconnect from the tcp socket."""
         self.debugpy_stream.socket.disconnect(self._get_endpoint())
         self.routing_id = None
         self.init_event = Event()
@@ -250,9 +284,11 @@ class DebugpyClient:
         self.wait_for_attach = True
 
     def receive_dap_frame(self, frame):
+        """Receive a dap frame."""
         self.message_queue.put_tcp_frame(frame)
 
     async def send_dap_request(self, msg):
+        """Send a dap request."""
         self._send_request(msg)
         if self.wait_for_attach and msg["command"] == "attach":
             rep = await self._handle_init_sequence()
@@ -266,6 +302,7 @@ class DebugpyClient:
 
 
 class Debugger:
+    """The debugger class."""
 
     # Requests that requires that the debugger has started
     started_debug_msg_types = [
@@ -284,6 +321,7 @@ class Debugger:
     def __init__(
         self, log, debugpy_stream, event_callback, shell_socket, session, just_my_code=True
     ):
+        """Initialize the debugger."""
         self.log = log
         self.debugpy_client = DebugpyClient(log, debugpy_stream, self._handle_event)
         self.shell_socket = shell_socket
@@ -353,6 +391,7 @@ class Debugger:
         return thread_name not in forbid_list
 
     async def handle_stopped_event(self):
+        """Handle a stopped event."""
         # Wait for a stopped event message in the stopped queue
         # This message is used for triggering the 'threads' request
         event = await self.stopped_queue.get()
@@ -368,6 +407,7 @@ class Debugger:
         return self.debugpy_client
 
     def start(self):
+        """Start the debugger."""
         if not self.debugpy_initialized:
             tmp_dir = get_tmp_directory()
             if not os.path.exists(tmp_dir):
@@ -397,6 +437,7 @@ class Debugger:
         return self.debugpy_initialized
 
     def stop(self):
+        """Stop the debugger."""
         self.debugpy_client.disconnect_tcp_socket()
 
         # Restore remove cleanup transformers
@@ -406,6 +447,7 @@ class Debugger:
             cleanup_transforms.insert(index, func)
 
     async def dumpCell(self, message):
+        """Handle a dump cell message."""
         code = message["arguments"]["code"]
         file_name = get_file_name(code)
 
@@ -422,11 +464,13 @@ class Debugger:
         return reply
 
     async def setBreakpoints(self, message):
+        """Handle a set breakpoints message."""
         source = message["arguments"]["source"]["path"]
         self.breakpoint_list[source] = message["arguments"]["breakpoints"]
         return await self._forward_message(message)
 
     async def source(self, message):
+        """Handle a source message."""
         reply = {"type": "response", "request_seq": message["seq"], "command": message["command"]}
         source_path = message["arguments"]["source"]["path"]
         if os.path.isfile(source_path):
@@ -441,6 +485,7 @@ class Debugger:
         return reply
 
     async def stackTrace(self, message):
+        """Handle a stack trace message."""
         reply = await self._forward_message(message)
         # The stackFrames array can have the following content:
         # { frames from the notebook}
@@ -462,6 +507,7 @@ class Debugger:
         return reply
 
     def accept_variable(self, variable_name):
+        """Accept a variable by name."""
         forbid_list = [
             "__name__",
             "__doc__",
@@ -490,6 +536,7 @@ class Debugger:
         return cond
 
     async def variables(self, message):
+        """Handle a variables message."""
         reply = {}
         if not self.stopped_threads:
             variables = self.variable_explorer.get_children_variables(
@@ -505,6 +552,7 @@ class Debugger:
         return reply
 
     async def attach(self, message):
+        """Handle an attach message."""
         host, port = self.debugpy_client.get_host_port()
         message["arguments"]["connect"] = {"host": host, "port": port}
         message["arguments"]["logToFile"] = True
@@ -517,6 +565,7 @@ class Debugger:
         return await self._forward_message(message)
 
     async def configurationDone(self, message):
+        """Handle a configuration done message."""
         reply = {
             "seq": message["seq"],
             "type": "response",
@@ -527,6 +576,7 @@ class Debugger:
         return reply
 
     async def debugInfo(self, message):
+        """Handle a debug info message."""
         breakpoint_list = []
         for key, value in self.breakpoint_list.items():
             breakpoint_list.append({"source": key, "breakpoints": value})
@@ -550,6 +600,7 @@ class Debugger:
         return reply
 
     async def inspectVariables(self, message):
+        """Handle an insepct variables message."""
         self.variable_explorer.untrack_all()
         # looks like the implementation of untrack_all in ptvsd
         # destroys objects we nee din track. We have no choice but
@@ -560,6 +611,7 @@ class Debugger:
         return self._build_variables_response(message, variables)
 
     async def richInspectVariables(self, message):
+        """Handle a rich inspect variables message."""
         reply = {
             "type": "response",
             "sequence_seq": message["seq"],
@@ -595,7 +647,7 @@ class Debugger:
                     "type": "request",
                     "command": "evaluate",
                     "seq": seq + 1,
-                    "arguments": {"expression": code, "frameId": frame_id},
+                    "arguments": {"expression": code, "frameId": frame_id, "context": "clipboard"},
                 }
             )
             if reply["success"]:
@@ -611,6 +663,7 @@ class Debugger:
         return reply
 
     async def modules(self, message):
+        """Handle a modules message."""
         modules = list(sys.modules.values())
         startModule = message.get("startModule", 0)
         moduleCount = message.get("moduleCount", len(modules))
@@ -625,6 +678,7 @@ class Debugger:
         return reply
 
     async def process_request(self, message):
+        """Process a request."""
         reply = {}
 
         if message["command"] == "initialize":
