@@ -26,7 +26,7 @@ from .kernelbase import _accepts_cell_id
 from .zmqshell import ZMQInteractiveShell
 
 try:
-    from IPython.core.interactiveshell import _asyncio_runner
+    from IPython.core.interactiveshell import _asyncio_runner  # type:ignore[attr-defined]
 except ImportError:
     _asyncio_runner = None
 
